import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.*;
import java.util.Hashtable;
import java.util.Enumeration;
import javax.swing.*;
import com.ibm.xml.parsers.DOMParser;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import ui.DOMTree;

/** SimpleTreeView3 builds on SimpleTreeView2
 *  to show how append the errors to a JTextArea and display them.
 */
public class SimpleTreeView3 extends JFrame {

    DOMParser parser;
    Errors errors;
    JTextArea messageText;

    public static void main (String args[]) {

        if (args.length > 0) {
            String filename = args[0];
            try {
                SimpleTreeView3 frame = new SimpleTreeView3(filename) ;
                frame.addWindowListener(new java.awt.event.WindowAdapter() {
                 public void windowClosing(java.awt.event.WindowEvent e) {
                     System.exit(0);
                 }
                });
                frame.setSize(600, 400);
                frame.setVisible(true);
                } catch (Exception e) {
                    e.printStackTrace(System.err);
                }
        }
    }

    public SimpleTreeView3 (String filename) {
        super("SimpleTreeView3: "+filename);
        try {
            parser = new DOMParser();
            errors = new Errors();
            parser.setNodeExpansion(DOMParser.FULL);
            parser.setErrorHandler(errors);
            parser.parse(filename);
            Document document = parser.getDocument();

            JPanel treePanel = new JPanel(new BorderLayout());      
            treePanel.add(new JScrollPane(new DOMTree(document)) {
                    public Dimension getPreferredSize(){
                        Dimension size = SimpleTreeView3.this.getSize();
                        return new Dimension(size.width, size.height/2 );
                    }
                }, BorderLayout.CENTER);

            treePanel.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("Tree View"),
                BorderFactory.createEmptyBorder(4, 4, 4, 4)
                ));
            
            // create the message panel 
            messageText = new JTextArea(3,5);
            JPanel messagePanel = new JPanel(new BorderLayout()); 
            messagePanel.add(new JScrollPane(messageText));
            messagePanel.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("Messages"),
                BorderFactory.createEmptyBorder(4, 4, 4, 4)
                ));

            // use the JSplitPane to allow user to dynamically resize...
            JComponent split = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                true, treePanel, messagePanel);

            getContentPane().add(split);
            
            Hashtable errorNodes = errors.getErrorNodes();
            Enumeration elements = errorNodes.elements();
            while (elements.hasMoreElements()) {
                //*** append errors to messageText
                messageText.append( (String)elements.nextElement());
            }    
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }
    }

    class Errors implements ErrorHandler {

        Hashtable errorNodes = new Hashtable();

        public void warning(SAXParseException ex) {
            store(ex, "[Warning]");
        }

        public void error(SAXParseException ex) {
            store(ex, "[Error]");
        }

        public void fatalError(SAXParseException ex) throws SAXException {
            store(ex, "[Fatal Error]");
        }

        public Hashtable getErrorNodes() {
            return errorNodes;
        }

        public Object getError(Node node) {
            return errorNodes.get(node);
        }

        public void clearErrors() {
            errorNodes.clear();
        }

        void store(SAXParseException ex, String type) {

            // build error text
            String errorString= type+" at line number, "+ex.getLineNumber()
                +": "+ex.getMessage()+"\n";

            Node currentNode = parser.getCurrentNode();
            if (currentNode == null) return;

            // accumulate any multiple errors per node in the Hashtable.
            String previous = (String) errorNodes.get(currentNode);
            if (previous != null)
                errorNodes.put(currentNode, previous +errorString);
            else
                errorNodes.put(currentNode, errorString);
        }
    }
}